@@ -7,11 +7,8 @@
   * Copyright 1995-2001 Bernd Schmidt
   */
 
-#define UAEMAJOR 0
-#define UAEMINOR 8
-#define UAESUBREV 25
-
-typedef enum { KBD_LANG_US, KBD_LANG_DK, KBD_LANG_DE, KBD_LANG_SE, KBD_LANG_FR, KBD_LANG_IT, KBD_LANG_ES } KbdLang;
+typedef enum { KBD_LANG_US, KBD_LANG_DK, KBD_LANG_DE, KBD_LANG_SE,
+	       KBD_LANG_FR, KBD_LANG_IT, KBD_LANG_ES } KbdLang;
 
 extern long int version;
 
@@ -22,6 +19,29 @@ struct strlist {
     char *str;
 };
 
+struct config_list {
+    const char *filename;
+    const char *description;
+    const char *sortstr;
+};
+
+extern struct config_list *predef_configs;
+extern int n_predef_configs;
+
+struct uae_rect {
+    int w, h;
+};
+
+extern struct uae_rect *gfx_fullscreen_modes, *gfx_windowed_modes;
+extern int n_fullscreen_modes, n_windowed_modes;
+
+#define FILTER_SOUND_OFF 0
+#define FILTER_SOUND_EMUL 1
+#define FILTER_SOUND_ON 2
+
+#define FILTER_SOUND_TYPE_A500 0
+#define FILTER_SOUND_TYPE_A1200 1
+
 /* maximum number native input devices supported (single type) */
 #define MAX_INPUT_DEVICES 6
 /* maximum number of native input device's buttons and axles supported */
@@ -40,14 +60,30 @@ struct uae_input_device {
     uae_u8 enabled;
 };
 
+typedef enum { DRV_NONE = -1, DRV_35_DD = 0, DRV_35_HD, DRV_525_SD, DRV_35_DD_ESCOM } drive_type;
+
+enum { CP_GENERIC = 1, CP_CDTV, CP_CD32, CP_A500, CP_A500P, CP_A600, CP_A1000,
+       CP_A1200, CP_A2000, CP_A3000, CP_A3000T, CP_A4000, CP_A4000T };
+
+struct gfx_params {
+    int width;
+    int height;
+    int lores;
+    int linedbl;
+    int correct_aspect;
+    int xcenter;
+    int ycenter;
+    int leds_on_screen;
+};
+
 struct uae_prefs {
     struct strlist *unknown_lines;
 
     char description[256];
+    char sortstr[256];
     char info[256];
 
     int illegal_mem;
-    int no_xhair;
     int use_serial;
     int serial_demand;
     int parallel_demand;
@@ -64,25 +100,18 @@ struct uae_prefs {
 
     int produce_sound;
     int sound_stereo;
-    int mixed_stereo;
-    int sound_bits;
+    int sound_stereo_separation;
+    int sound_mixed_stereo_delay;
     int sound_freq;
-    int sound_minbsiz;
     int sound_maxbsiz;
-    int sound_pri_time;
-    int sound_pri_cutoff;
     int sound_interpol;
+    int sound_filter;
+    int sound_filter_type;
 
     int gfx_framerate;
-    int gfx_width;
-    int gfx_height;
-    int gfx_lores;
-    int gfx_linedbl;
-    int gfx_correct_aspect;
+    struct gfx_params gfx_w, gfx_f;
     int gfx_afullscreen;
     int gfx_pfullscreen;
-    int gfx_xcenter;
-    int gfx_ycenter;
     int color_mode;
 
     int blits_32bit_enabled;
@@ -90,10 +119,16 @@ struct uae_prefs {
     unsigned int chipset_mask;
     int ntscmode;
     int collision_level;
-    int leds_on_screen;
+
+    int cs_rtc;
+    int cs_ide;
+    int cs_a1000ram;
+    int cs_fatgaryrev;
+    int cs_ramseyrev;
 
     char df[4][256];
     char romfile[256];
+    unsigned int rom_crc32;
     char romextfile[256];
     char keyfile[256];
     char prtname[256];
@@ -104,23 +139,25 @@ struct uae_prefs {
     char path_rom[256];
 
     int m68k_speed;
-    int cpu_level;
-    int cpu_compatible;
+    int cpu_model;
+    int fpu_model;
     int address_space_24;
 
     uae_u32 z3fastmem_size;
     uae_u32 fastmem_size;
     uae_u32 chipmem_size;
     uae_u32 bogomem_size;
-    uae_u32 a3000mem_size;
+    uae_u32 mbresmem_low_size;
+    uae_u32 mbresmem_high_size;
     uae_u32 gfxmem_size;
 
     int kickshifter;
 
     struct uaedev_mount_info *mountinfo;
+    int bootrom;
 
     int nr_floppies;
-    int dfxtype[4];
+    drive_type dfxtype[4];
 
     /* Target specific options */
     int x11_use_low_bandwidth;
@@ -153,21 +190,22 @@ struct uae_prefs {
 
 /* Contains the filename of .uaerc */
 extern char optionsfile[];
-extern void save_options (FILE *, struct uae_prefs *);
+extern void save_options (FILE *, const struct uae_prefs *);
 
 extern void default_prefs (struct uae_prefs *);
 extern void discard_prefs (struct uae_prefs *);
 
-extern int parse_cmdline_option (char, char *);
+extern int parse_cmdline_option (struct uae_prefs *, char, char *);
 
-extern int cfgfile_yesno (char *option, char *value, char *name, int *location);
-extern int cfgfile_intval (char *option, char *value, char *name, int *location, int scale);
-extern int cfgfile_strval (char *option, char *value, char *name, int *location, const char *table[], int more);
-extern int cfgfile_string (char *option, char *value, char *name, char *location, int maxsz);
+extern int cfgfile_yesno (const char *option, const char *value, const char *name, int *location);
+extern int cfgfile_uintval (const char *option, const char *value, const char *name, unsigned int *location, int scale);
+extern int cfgfile_intval (const char *option, const char *value, const char *name, int *location, int scale);
+extern int cfgfile_strval (const char *option, const char *value, const char *name, int *location, const char *table[], int more);
+extern int cfgfile_string (const char *option, const char *value, const char *name, char *location, int maxsz);
 extern char *cfgfile_subst_path (const char *path, const char *subst, const char *file);
 
-extern int target_parse_option (struct uae_prefs *, char *option, char *value);
-extern void target_save_options (FILE *, struct uae_prefs *);
+extern int target_parse_option (struct uae_prefs *, const char *option, const char *value);
+extern void target_save_options (FILE *, const struct uae_prefs *);
 
 extern int cfgfile_load (struct uae_prefs *, const char *filename);
 extern int cfgfile_save (struct uae_prefs *, const char *filename);
@@ -176,7 +214,11 @@ extern int cfgfile_parse_option (struct uae_prefs *p, char *option, char *value)
 extern int cfgfile_get_description (const char *filename, char *description);
 extern void cfgfile_show_usage (void);
 
-extern void fixup_prefs_dimensions (struct uae_prefs *prefs);
+extern int cstype_from_prefs (struct uae_prefs *p);
+extern void built_in_chipset_prefs (struct uae_prefs *p, int);
+
+extern int fixup_prefs_dimensions (struct gfx_params *, struct uae_rect *, int);
+extern void fixup_cpu (struct uae_prefs *);
 
 extern void check_prefs_changed_custom (void);
 extern void check_prefs_changed_cpu (void);
@@ -185,6 +227,8 @@ extern int check_prefs_changed_gfx (void);
 
 extern struct uae_prefs currprefs, changed_prefs;
 
+extern struct gfx_params *curr_gfx;
+
 extern void machdep_init (void);
 
 /* AIX doesn't think it is Unix. Neither do I. */
