@@ -10,7 +10,6 @@
 #include "sysdeps.h"
 #include <assert.h>
 
-#include "config.h"
 #include "options.h"
 #include "threaddep/thread.h"
 #include "uae.h"
@@ -30,21 +29,22 @@
 #include "gui.h"
 #include "zfile.h"
 #include "autoconf.h"
+#include "filesys.h"
 #include "osemu.h"
-#include "osdep/exectasks.h"
 #include "picasso96.h"
+#include "traps.h"
 #include "bsdsocket.h"
 #include "uaeexe.h"
 #include "native2amiga.h"
 #include "scsidev.h"
+#include "romlist.h"
 
 #ifdef USE_SDL
 #include "SDL.h"
 #endif
 
-long int version = 256*65536L*UAEMAJOR + 65536L*UAEMINOR + UAESUBREV;
-
 struct uae_prefs currprefs, changed_prefs;
+struct gfx_params *curr_gfx;
 
 int no_gui = 0;
 int joystickpresent = 0;
@@ -76,6 +76,99 @@ char *colormodes[] = { "256 colors", "32768 colors", "65536 colors",
     "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", ""
 };
 
+struct uae_rect *gfx_fullscreen_modes, *gfx_windowed_modes;
+int n_fullscreen_modes, n_windowed_modes;
+
+static struct uae_rect default_windowed_modes[] =
+{
+    { 320, 256 },
+    { 400, 300 },
+    { 640, 512 },
+    { 800, 600 }
+};
+
+struct config_list *predef_configs;
+int n_predef_configs;
+int predef_configs_space;
+
+static int sortfn (const void *a, const void *b)
+{
+    struct config_list *pa = (struct config_list *)a;
+    struct config_list *pb = (struct config_list *)b;
+    return strcmp (pa->sortstr, pb->sortstr);
+}
+
+static void scan_configs (const char *path)
+{
+    DIR *dir;
+    int pathlen = strlen (path);
+    int bufsz = pathlen + 256;
+    char *buffer;
+    uae_u8 *data;
+
+    predef_configs_space = 20;
+    predef_configs = malloc (sizeof (struct config_list) * 20);
+    n_predef_configs = 0;
+
+    dir = opendir (path);
+    if (!dir)
+	return;
+
+    buffer = malloc (bufsz);
+    if (!buffer)
+	goto out;
+    data = malloc (1024 * 1024);
+    if (!data)
+	goto out1;
+
+    strcpy (buffer, path);
+    buffer[pathlen++] = '/';
+    buffer[pathlen] = '\0';
+    for (;;) {
+	struct uae_prefs p;
+	struct zfile *f;
+	struct dirent *ent = readdir (dir);
+	int len;
+	struct romdata *rd;
+	long size;
+
+	if (!ent)
+	    break;
+
+	len = strlen (ent->d_name);
+	if (len + pathlen + 1 >= bufsz) {
+	    bufsz = len + pathlen + 200;
+	    buffer = realloc (buffer, bufsz);
+	    if (!buffer) {
+		free (data);
+		goto out;
+	    }
+	}
+	strcpy (buffer + pathlen, ent->d_name);
+
+	if (n_predef_configs >= predef_configs_space) {
+	    predef_configs_space += 20;
+	    predef_configs = realloc (predef_configs,
+				      sizeof (struct config_list) * predef_configs_space);
+	}
+	default_prefs (&p);
+	strcpy (p.description, "");
+	strcpy (p.sortstr, "");
+	cfgfile_load (&p, buffer);
+	if (strlen (p.description) > 0) {
+	    predef_configs[n_predef_configs].filename = strdup (buffer);
+	    predef_configs[n_predef_configs].description = strdup (p.description);
+	    predef_configs[n_predef_configs++].sortstr = strdup (p.sortstr);
+	}
+    }
+    qsort (predef_configs, n_predef_configs, sizeof *predef_configs, sortfn);
+    free (data);
+  out1:
+    free (buffer);
+  out:
+    closedir (dir);
+}
+
 void discard_prefs (struct uae_prefs *p)
 {
     struct strlist **ps = &p->unknown_lines;
@@ -94,13 +187,11 @@ void default_prefs (struct uae_prefs *p)
 
     p->start_gui = 1;
     p->start_debugger = 0;
-    p->leds_on_screen = 1;
 
     p->unknown_lines = 0;
     /* Note to porters: please don't change any of these options! UAE is supposed
      * to behave identically on all platforms if possible. */
     p->illegal_mem = 0;
-    p->no_xhair = 0;
     p->use_serial = 0;
     p->serial_demand = 0;
     p->parallel_demand = 0;
@@ -112,24 +203,29 @@ void default_prefs (struct uae_prefs *p)
     p->emul_accuracy = 2;
     p->test_drawing_speed = 0;
 
-    p->produce_sound = 0;
+    p->produce_sound = 3;
     p->sound_stereo = 0;
-    p->sound_bits = DEFAULT_SOUND_BITS;
+    p->sound_stereo_separation = 7;
+    p->sound_mixed_stereo_delay = 0;
     p->sound_freq = DEFAULT_SOUND_FREQ;
-    p->sound_minbsiz = DEFAULT_SOUND_MINB;
     p->sound_maxbsiz = DEFAULT_SOUND_MAXB;
-    p->sound_interpol = 0;
+    p->sound_interpol = 2;
+    p->sound_filter = FILTER_SOUND_OFF;
+    p->sound_filter_type = FILTER_SOUND_TYPE_A500;
 
     p->gfx_framerate = 1;
-    p->gfx_width = 800;
-    p->gfx_height = 600;
-    p->gfx_lores = 0;
-    p->gfx_linedbl = 2;
+    p->gfx_w.width = 800;
+    p->gfx_w.height = 600;
+    p->gfx_w.lores = 0;
+    p->gfx_w.linedbl = 2;
+    p->gfx_w.correct_aspect = 0;
+    p->gfx_w.xcenter = 0;
+    p->gfx_w.ycenter = 0;
+    p->gfx_f = p->gfx_w;
+    p->gfx_w.leds_on_screen = 0;
+    p->gfx_f.leds_on_screen = 1;
     p->gfx_afullscreen = 0;
     p->gfx_pfullscreen = 0;
-    p->gfx_correct_aspect = 0;
-    p->gfx_xcenter = 0;
-    p->gfx_ycenter = 0;
     p->color_mode = 0;
 
     p->x11_use_low_bandwidth = 0;
@@ -148,6 +244,13 @@ void default_prefs (struct uae_prefs *p)
 
     p->immediate_blits = 0;
     p->collision_level = 1;
+ 
+    p->chipset_mask = CSMASK_ECS_AGNUS;
+    p->cs_rtc = 2;
+    p->cs_a1000ram = 0;
+    p->cs_fatgaryrev = -1;
+    p->cs_ramseyrev = -1;
+    p->cs_ide = 0;
 
     strcpy (p->df[0], "df0.adf");
     strcpy (p->df[1], "df1.adf");
@@ -157,6 +260,7 @@ void default_prefs (struct uae_prefs *p)
     strcpy (p->romfile, "kick.rom");
     strcpy (p->keyfile, "");
     strcpy (p->prtname, DEFPRTNAME);
+    p->rom_crc32 = 0;
 
     strcpy (p->path_rom, "./");
     strcpy (p->path_floppy, "./");
@@ -165,134 +269,161 @@ void default_prefs (struct uae_prefs *p)
     strcpy (p->prtname, "");
     strcpy (p->sername, "");
 
+    p->nr_floppies = 2;
+    p->dfxtype[0] = DRV_35_DD;
+    p->dfxtype[1] = DRV_35_DD;
+    p->dfxtype[2] = DRV_NONE;
+    p->dfxtype[3] = DRV_NONE;
+
     p->m68k_speed = 0;
-    p->cpu_level = 2;
-    p->cpu_compatible = 0;
+    p->cpu_model = 68020;
+    p->fpu_model = 0;
     p->address_space_24 = 0;
 
     p->fastmem_size = 0x00000000;
-    p->a3000mem_size = 0x00000000;
+    p->mbresmem_low_size = 0x00000000;
+    p->mbresmem_high_size = 0x00000000;
     p->z3fastmem_size = 0x00000000;
     p->chipmem_size = 0x00200000;
     p->bogomem_size = 0x00000000;
     p->gfxmem_size = 0x00000000;
 
-    p->nr_floppies = 4;
-
     p->mountinfo = alloc_mountinfo ();
     inputdevice_default_prefs (p);
+    p->bootrom = 1;
 }
 
-void fixup_prefs_dimensions (struct uae_prefs *prefs)
+int fixup_prefs_dimensions (struct gfx_params *p, struct uae_rect *modes, int n_modes)
 {
-    if (prefs->gfx_width < 320)
-	prefs->gfx_width = 320;
-    if (prefs->gfx_height < 200)
-	prefs->gfx_height = 200;
-    if (prefs->gfx_height > 300 && ! prefs->gfx_linedbl)
-	prefs->gfx_height = 300;
-    if (prefs->gfx_height > 600)
-	prefs->gfx_height = 600;
-
-    prefs->gfx_width += 7; /* X86.S wants multiples of 4 bytes, might be 8 in the future. */
-    prefs->gfx_width &= ~7;
+    int i;
+    if (p->width < 320)
+	p->width = 320;
+    if (p->height < 200)
+	p->height = 200;
+    for (i = 0; i < n_modes; i++) {
+	if ((modes[i].w >= p->width && modes[i].h >= p->height)
+	    || i == n_modes - 1)
+	{
+	    p->width = modes[i].w;
+	    p->height = modes[i].h;
+	    p->lores = p->width <= 512;
+	    if (p->height > 300 && !p->linedbl)
+		p->linedbl = 1;
+	    return i;
+	}
+    }
+    return n_modes - 1;
 }
 
-static void fix_options (void)
+void fixup_cpu (struct uae_prefs *p)
+{
+}
+
+void fixup_sound (struct uae_prefs *p)
+{
+    if (p->sound_stereo_separation < 0)
+	p->sound_stereo_separation = 0;
+    if (p->sound_stereo_separation > MIXED_STEREO_MAX)
+	p->sound_stereo_separation = MIXED_STEREO_MAX;
+}
+
+static void fixup_prefs (struct uae_prefs *p)
 {
     int err = 0;
 
-    if ((currprefs.chipmem_size & (currprefs.chipmem_size - 1)) != 0
-	|| currprefs.chipmem_size < 0x80000
-	|| currprefs.chipmem_size > 0x800000)
+    if ((p->chipmem_size & (p->chipmem_size - 1)) != 0
+	|| p->chipmem_size < 0x80000
+	|| p->chipmem_size > 0x800000)
     {
-	currprefs.chipmem_size = 0x200000;
+	p->chipmem_size = 0x200000;
 	write_log ("Unsupported chipmem size!\n");
 	err = 1;
     }
-    if ((currprefs.fastmem_size & (currprefs.fastmem_size - 1)) != 0
-	|| (currprefs.fastmem_size != 0 && (currprefs.fastmem_size < 0x100000 || currprefs.fastmem_size > 0x800000)))
+    if ((p->fastmem_size & (p->fastmem_size - 1)) != 0
+	|| (p->fastmem_size != 0 && (p->fastmem_size < 0x100000 || p->fastmem_size > 0x800000)))
     {
-	currprefs.fastmem_size = 0;
+	p->fastmem_size = 0;
 	write_log ("Unsupported fastmem size!\n");
 	err = 1;
     }
-    if ((currprefs.gfxmem_size & (currprefs.gfxmem_size - 1)) != 0
-	|| (currprefs.gfxmem_size != 0 && (currprefs.gfxmem_size < 0x100000 || currprefs.gfxmem_size > 0x2000000)))
+    if ((p->gfxmem_size & (p->gfxmem_size - 1)) != 0
+	|| (p->gfxmem_size != 0 && (p->gfxmem_size < 0x100000 || p->gfxmem_size > 0x2000000)))
     {
-	write_log ("Unsupported graphics card memory size %lx!\n", currprefs.gfxmem_size);
-	currprefs.gfxmem_size = 0;
+	write_log ("Unsupported graphics card memory size %lx!\n", p->gfxmem_size);
+	p->gfxmem_size = 0;
 	err = 1;
     }
-    if ((currprefs.z3fastmem_size & (currprefs.z3fastmem_size - 1)) != 0
-	|| (currprefs.z3fastmem_size != 0 && (currprefs.z3fastmem_size < 0x100000 || currprefs.z3fastmem_size > 0x4000000)))
+    if ((p->z3fastmem_size & (p->z3fastmem_size - 1)) != 0
+	|| (p->z3fastmem_size != 0 && (p->z3fastmem_size < 0x100000 || p->z3fastmem_size > 0x4000000)))
     {
-	currprefs.z3fastmem_size = 0;
+	p->z3fastmem_size = 0;
 	write_log ("Unsupported Zorro III fastmem size!\n");
 	err = 1;
     }
-    if (currprefs.address_space_24 && (currprefs.gfxmem_size != 0 || currprefs.z3fastmem_size != 0)) {
-	currprefs.z3fastmem_size = currprefs.gfxmem_size = 0;
+    if (p->address_space_24 && (p->gfxmem_size != 0 || p->z3fastmem_size != 0)) {
+	p->z3fastmem_size = p->gfxmem_size = 0;
 	write_log ("Can't use a graphics card or Zorro III fastmem when using a 24 bit\n"
 		 "address space - sorry.\n");
     }
-    if ((currprefs.bogomem_size & (currprefs.bogomem_size - 1)) != 0
-	|| (currprefs.bogomem_size != 0 && (currprefs.bogomem_size < 0x80000 || currprefs.bogomem_size > 0x100000)))
+    if ((p->bogomem_size & (p->bogomem_size - 1)) != 0
+	|| (p->bogomem_size != 0 && (p->bogomem_size < 0x80000 || p->bogomem_size > 0x100000)))
     {
-	currprefs.bogomem_size = 0;
+	p->bogomem_size = 0;
 	write_log ("Unsupported bogomem size!\n");
 	err = 1;
     }
 
-    if (currprefs.chipmem_size > 0x200000 && currprefs.fastmem_size != 0) {
+    if (p->chipmem_size > 0x200000 && p->fastmem_size != 0) {
 	write_log ("You can't use fastmem and more than 2MB chip at the same time!\n");
-	currprefs.fastmem_size = 0;
+	p->fastmem_size = 0;
 	err = 1;
     }
 #if 0
-    if (currprefs.m68k_speed < -1 || currprefs.m68k_speed > 20) {
+    if (p->m68k_speed < -1 || p->m68k_speed > 20) {
 	write_log ("Bad value for -w parameter: must be -1, 0, or within 1..20.\n");
-	currprefs.m68k_speed = 4;
+	p->m68k_speed = 4;
 	err = 1;
     }
 #endif
 
-    if (currprefs.produce_sound < 0 || currprefs.produce_sound > 3) {
+    if (p->produce_sound < 0 || p->produce_sound > 3) {
 	write_log ("Bad value for -S parameter: enable value must be within 0..3\n");
-	currprefs.produce_sound = 0;
+	p->produce_sound = 0;
 	err = 1;
     }
-    if (currprefs.cpu_level < 2 && currprefs.z3fastmem_size > 0) {
+    if (p->cpu_model < 68020 && p->z3fastmem_size > 0) {
 	write_log ("Z3 fast memory can't be used with a 68000/68010 emulation. It\n"
 		 "requires a 68020 emulation. Turning off Z3 fast memory.\n");
-	currprefs.z3fastmem_size = 0;
+	p->z3fastmem_size = 0;
 	err = 1;
     }
-    if (currprefs.gfxmem_size > 0 && (currprefs.cpu_level < 2 || currprefs.address_space_24)) {
+    if (p->gfxmem_size > 0 && (p->cpu_model < 68020 || p->address_space_24)) {
 	write_log ("Picasso96 can't be used with a 68000/68010 or 68EC020 emulation. It\n"
 		 "requires a 68020 emulation. Turning off Picasso96.\n");
-	currprefs.gfxmem_size = 0;
+	p->gfxmem_size = 0;
 	err = 1;
     }
-#ifndef BSDSOCKET_SUPPORTED
-    if (currprefs.socket_emu) {
+#ifndef BSDSOCKET
+    if (p->socket_emu) {
 	write_log ("Compile-time option of BSDSOCKET_SUPPORTED was not enabled.  You can't use bsd-socket emulation.\n");
-	currprefs.socket_emu = 0;
+	p->socket_emu = 0;
 	err = 1;
     }
 #endif
 
-    if (currprefs.nr_floppies < 1 || currprefs.nr_floppies > 4) {
+    if (p->nr_floppies < 1 || p->nr_floppies > 4) {
 	write_log ("Invalid number of floppies.  Using 4.\n");
-	currprefs.nr_floppies = 4;
+	p->nr_floppies = 4;
 	err = 1;
     }
-    if (currprefs.collision_level < 0 || currprefs.collision_level > 3) {
+    if (p->collision_level < 0 || p->collision_level > 3) {
 	write_log ("Invalid collision support level.  Using 1.\n");
-	currprefs.collision_level = 1;
+	p->collision_level = 1;
 	err = 1;
     }
 
+    fixup_sound (p);
+
     if (err)
 	write_log ("Please use \"uae -h\" to get usage information.\n");
 }
@@ -346,7 +477,7 @@ void parse_cmdline (int argc, char **argv)
 		int extra_arg = *arg == '\0';
 		if (extra_arg)
 		    arg = i + 1 < argc ? argv[i + 1] : 0;
-		if (parse_cmdline_option (argv[i][1], arg) && extra_arg)
+		if (parse_cmdline_option (&currprefs, argv[i][1], arg) && extra_arg)
 		    i++;
 	    }
 	}
@@ -373,7 +504,10 @@ static void parse_cmdline_and_init_file (int argc, char **argv)
 
     if (! cfgfile_load (&currprefs, optionsfile)) {
 #ifdef OPTIONS_IN_HOME
-	/* sam: if not found in $HOME then look in current directory */
+	/* If not found in $HOME then look in current directory.  However,
+	 * don't use the optionsfile variable, so that we will save changes
+	 * to the home directory.  */
+	char pwd_optionsfile[256];
 	strcpy (optionsfile, OPTIONSFILENAME);
 	cfgfile_load (&currprefs, optionsfile);
 #endif
@@ -453,6 +587,15 @@ void real_main (int argc, char **argv)
 
     default_prefs (&currprefs);
 
+#ifdef SYSTEM_CFGDIR
+    scan_configs (SYSTEM_CFGDIR);
+#endif
+
+    /* Can be overriden in graphics_setup, although there's not much of a
+       point.  Fullscreen modes are filled in by graphics_setup.  */
+    gfx_windowed_modes = default_windowed_modes;
+    n_windowed_modes = sizeof default_windowed_modes / sizeof *default_windowed_modes;
+
     if (! graphics_setup ()) {
 	exit (1);
     }
@@ -464,6 +607,7 @@ void real_main (int argc, char **argv)
     parse_cmdline_and_init_file (argc, argv);
 
     machdep_init ();
+    init_gtod ();
 
     if (! setup_sound ()) {
 	write_log ("Sound driver unavailable: Sound output disabled\n");
@@ -474,7 +618,7 @@ void real_main (int argc, char **argv)
     changed_prefs = currprefs;
     no_gui = ! currprefs.start_gui;
     if (! no_gui) {
-	int err = gui_init ();
+	int err = gui_init (1);
 	currprefs = changed_prefs;
 	if (err == -1) {
 	    write_log ("Failed to initialize the GUI\n");
@@ -487,9 +631,13 @@ void real_main (int argc, char **argv)
 	currprefs.produce_sound = 0;
     }
 
-    fix_options ();
+    fixup_prefs (&currprefs);
     changed_prefs = currprefs;
 
+#ifdef SYSTEM_ROMDIR
+    scan_roms (SYSTEM_ROMDIR, ROMLOC_SYSTEM);
+#endif
+    scan_roms (currprefs.path_rom, ROMLOC_USER);
     /* Install resident module to get 8MB chipmem, if requested */
     rtarea_setup ();
 
@@ -499,7 +647,7 @@ void real_main (int argc, char **argv)
     memory_init ();
 
     filesys_install ();
-    gfxlib_install ();
+    bsdlib_install ();
     emulib_install ();
     uaeexe_install ();
     native2amiga_install ();
@@ -514,6 +662,7 @@ void real_main (int argc, char **argv)
     gui_update ();
 
     if (graphics_init ()) {
+	reset_drawing ();
 	setup_brkhandler ();
 	if (currprefs.start_debugger && debuggable ())
 	    activate_debugger ();
@@ -523,6 +672,12 @@ void real_main (int argc, char **argv)
     leave_program ();
 }
 
+void uae_abort (const char *msg)
+{
+    write_log ("%s", msg);
+    abort ();
+}
+
 #ifndef NO_MAIN_IN_MAIN_C
 int main (int argc, char **argv)
 {
