@@ -11,17 +11,19 @@ extern uae_u16 *sndbufpt;
 extern int sound_fd;
 extern int sndbufsize;
 
-static __inline__ void flush_sound_buffer (void)
+STATIC_INLINE void flush_sound_buffer (void)
 {
     write (sound_fd, sndbuffer, sndbufsize);
     sndbufpt = sndbuffer;
 }
 
-static __inline__ void check_sound_buffers (void)
+STATIC_INLINE void check_sound_buffers (void)
 {
     if ((char *)sndbufpt - (char *)sndbuffer >= sndbufsize) {
 	flush_sound_buffer();
+	return 1;
     }
+    return 0;
 }
 
 #define PUT_SOUND_BYTE(b) do { *(uae_u8 *)sndbufpt = b; sndbufpt = (uae_u16 *)(((uae_u8 *)sndbufpt) + 1); } while (0)
